
part of 'peter_package_demo.dart';      //属于哪个组

const Color weChatThemeColor = Color.fromRGBO(220, 220, 220, 1.0);              //主题色：命名以小写字母开始
double screenWidth(BuildContext context) => MediaQuery.of(context).size.width;  //屏幕宽度
double screenHeight(BuildContext context) => MediaQuery.of(context).size.height;//屏幕高度
const INDEX_WORDS = [
  '🔍',
  '☆',
  'A',
  'B',
  'C',
  'D',
  'E',
  'F',
  'G',
  'H',
  'I',
  'J',
  'K',
  'L',
  'M',
  'N',
  'O',
  'P',
  'Q',
  'R',
  'S',
  'T',
  'U',
  'V',
  'W',
  'X',
  'Y',
  'Z'
];

/**
 * 索引条相关的功能代码
 */

class IndexBar extends StatefulWidget {
  final void Function(String str)? indexBarCallBack; //回调函数，有一个参数 - 将点击的index回调出去
  const IndexBar({this.indexBarCallBack,Key? key}) : super(key:key);

  @override
  _IndexBarState createState() => _IndexBarState();
}

class _IndexBarState extends State<IndexBar> {
  Color _bgColor = Color.fromRGBO(1, 1, 1, 0);
  Color _textColor = Colors.black;

  //ListView滚动需要的变量
  double _indicatorY = 0.0; //y坐标 -1.1 ~ 1.1
  String _indicatorText = 'A'; //指示器显示的索引值
  bool _indicatorHidden = true; //指示器是否需要改变

  @override
  void initState() {
    //生命周期方法，页面被渲染(载入)的时候会进来 (需要重新运行) - 从通讯录切换到发现页面，通讯录页面会被销毁
    super.initState();
  }

  int getIndex(BuildContext context, Offset globalPosition) {
    RenderBox box = context.findRenderObject() as RenderBox; //拿到当前小部件的盒子(强制类型转换)
    Offset nowOffset = box.globalToLocal(globalPosition); //将点击位置转换为部件内的点
    double y = nowOffset.dy; //拿到y值
    var itemHeight = screenHeight(context) / 2 / INDEX_WORDS.length; //每一个字符占用的高度
    int index = (y ~/ itemHeight).clamp(0, INDEX_WORDS.length - 1); //当前点击的是哪一个item(限定最后值的范围)
    print('选中了：${INDEX_WORDS[index]}');
    return index;
  }

  @override
  Widget build(BuildContext context) {
    final List<Widget> words = [];
    for (int i = 0; i < INDEX_WORDS.length; i++) {
      //与界面相关的处理放在build里面
      words.add(Expanded(child: Text(INDEX_WORDS[i], style: TextStyle(fontSize: 10, color: _textColor),))); //words里面装的是组件(如果不用Expanded,占不满)
    }

    return Positioned(
      top: screenHeight(context) / 8,
      right: 0,
      width: 120,
      height: screenHeight(context) / 2,
      child: Row(
        children: [
          Container(
            alignment: Alignment(0, _indicatorY), //通过修改Y值可以改变整个Stack的位置 -1.1 ~ 1.1
            width: 100,
            // color: Colors.red,
            child: _indicatorHidden
                ? null
                : Stack(
              alignment: Alignment(-0.2, 0), //定位(尝试出来的数据)
              children: [
                Image(
                  image: AssetImage('images/bubble.png',package: 'peter_package_demo'),
                  width: 60,
                ),
                Text(
                  _indicatorText,
                  style: TextStyle(fontSize: 35, color: Colors.white),
                ),
              ],
            ),
          ),
          GestureDetector(
            onVerticalDragDown: (DragDownDetails details) {
              //拖拽开始
              // print('当前点击位置：${details.globalPosition}');    //整个屏幕中的位置
              // print('当前组件中的位置: ${details.localPosition}');  //在当前组件中的哪个位置
              int index = getIndex(context, details.globalPosition);
              String str = INDEX_WORDS[index];
              if (widget.indexBarCallBack != null) {
                widget.indexBarCallBack!(str);
              }
              setState(() {
                _bgColor = Color.fromRGBO(1, 1, 1, 0.5);
                _textColor = Colors.white;
                _indicatorY = 2.2 / INDEX_WORDS.length * index - 1.1; //从 -1.1 开始的，到 1.1 结束
                print('index = $index  y = $_indicatorY');
                _indicatorText = str;
                _indicatorHidden = false;
              });
            },
            onVerticalDragUpdate: (DragUpdateDetails details) {
              int index = getIndex(context, details.globalPosition);
              String str = INDEX_WORDS[index];
              setState(() {
                _indicatorY = 2.2 / INDEX_WORDS.length * index - 1.1; //从 -1.1 开始的，到 1.1 结束
                _indicatorText = str;
                _indicatorHidden = false;
              });
              // print('选中了：$str');
              if (widget.indexBarCallBack != null) {
                //注意：使用之前要加上非空的判断，否则可能会有异常
                widget.indexBarCallBack!(str);
              }
            },
            onVerticalDragEnd: (DragEndDetails details) {
              setState(() {
                _bgColor = Color.fromRGBO(1, 1, 1, 0);
                _textColor = Colors.black;
                _indicatorHidden = true;
              });
            },
            child: Container(
              width: 20,
              color: _bgColor,
              child: Column(
                children: words,
              ),
            ),
          ),
        ],
      ),
    );
  }
}
